# Tutorial: Use `armarx_armem` to load data from Long-Term Memory

## Record Data using the Long-Term Memory
To record data using the long-term memory see the How-To in the `Robot_API` section.

## Loading a full Memory

To load data from the file version of the long-term memory you need to know the following things:
- the absolute path to your exported memories
- the name of the export

If you have this information you can use the functionality of a `MemoryServer` to load all references from that memory.
This only loads references, not the actual content yet, for efficiency reasons.
You can use the following code snippet to load the references for a memory located at `testData/2024_07_16/LTM`:

```python
from armarx_memory.ltm.memory_server import MemoryServer

def loadReferencesForServer():
    base_path = "testData/2024_07_16"
    export_name = "LTM"
    server = MemoryServer(base_path, export_name)
    server.loadReferences()
    return server
```

## Loading an Entity Snapshot

You probably want the content of a specific (or all) Entity Snapshots and their instances to work with the data in your code.
To do so, you need to know the following things:
- the name of the memory servers (e.g. `RobotState`)
- the name of the core segment (e.g. `Localization`)
- the name of the provider segment (e.g. `Armar3`)
- the name of the entity (e.g. `Map,Odom`)

From this information you can then build a MemoryID, which can then be used to retrieve the entity using the `MemoryServer` (from the code snippet above).
This loads the actual content of the entity instead of just the references.

The entity contains all EntitySnapshots with their EntityInstances that were part of this recording.
To get a specific `EntitySnapshot` you need to know the SnapshotID of this specific snapshot. (Or you can iterate over all snapshots/SnapshotIDs if you are interested in all snapshots).

In the example below we just care for the first snapshot and the first instance of this snapshot.
But SnapshotIDs correspond to their timestamps so you can also get the last snapshot or the first five etc. by using the sorted list of SnapshotIDs.

The example loads the data (and metadata) of the chosen instance, converts it's data to an `AronDict` (all instances are an `AronDict`in the first layer) and 
accesses the individual elements of the data and chooses the `transform` element:

```python
from armarx_memory.core.MemoryID import MemoryID
from armarx_memory.ltm.base.entity_snapshot import EntitySnapshot
from armarx_memory.ltm.base.entity_instance import EntityInstance

from armarx_memory.aron.data.variant import AronDict, AronNDArray

def robotStateLoading():
    server = loadReferencesForServer()  # see code snippet from before
    mid = MemoryID("RobotState", "Localization", "Armar3", "Map,Odom")
    entity = server.get_entity(mid)
    snapshot_id = list(entity.snapshots.keys())[0]
    snapshot: EntitySnapshot = entity.snapshots[snapshot_id]
    instance: EntityInstance = snapshot.instances[0]
    instance.load()
    instance_data: AronDict = instance.data
    transform_data: AronNDArray = instance_data.elements['transform']
```

### Loading metadata

To load the metadata associated with an `EntityInstance` you can use the following code snippet:

```python
from armarx_memory.core.MemoryID import MemoryID
from armarx_memory.ltm.base.entity_snapshot import EntitySnapshot
from armarx_memory.ltm.base.entity_instance import EntityInstance

def loadingMetadata():
    server = loadReferencesForServer()
    mid_vision = MemoryID("Vision", "ImageRGB", "Armar3WideImageProvider", "images")
    entity = server.get_entity(mid_vision)
    snapshot_id = list(entity.snapshots.keys())[0]
    snapshot: EntitySnapshot = entity.snapshots[snapshot_id]
    instance: EntityInstance = snapshot.instances[0]
    instance.load()
    metadata = instance.metadata
    return metadata
```

This example uses an `EntityInstance` of the `VisionMemory` memory server.

## Using the loaded data in your code

Be aware that the data loaded using these methods is in the form of aron objects. You can either continue using these or convert them into standard objects (like integers etc.) using the aron functionality.
